#!/usr/bin/fades

import argparse
import datetime
import logging
import os
import webbrowser

from urllib import request

import exifread  # fades
import flickrapi  # fades
import yaml  # fades


# set up logging
logger = logging.getLogger("msg")
h = logging.StreamHandler()
h.setFormatter(logging.Formatter("%(asctime)s %(levelname)-12s %(message)s"))
logger.addHandler(h)
logger.setLevel(logging.INFO)


def main(config_path, dirpath):
    """Main entry point."""
    with open(config_path, 'rt', encoding='ascii') as fh:
        config = yaml.load(fh)

    # auth
    logger.info("Authenticating...")
    flickr = flickrapi.FlickrAPI(config['api_key'], config['sharedsecret'], format='parsed-json')
    if not flickr.token_valid(perms='delete'):
        # get a request token
        flickr.get_request_token(oauth_callback='oob')

        # ppen a browser at the authentication URL. Do this however
        # you want, as long as the user visits that URL.
        authorize_url = flickr.auth_url(perms='delete')
        webbrowser.open_new_tab(authorize_url)

        # get the verifier code from the user. Do this however you
        # want, as long as the user gives the application the code.
        verifier = input('Verifier code from the web page just opened: ')

        # trade the request token for an access token
        flickr.get_access_token(verifier)

    logger.info("Getting photo list from album...")
    page = 1
    all_photos = []
    while True:
        resp = flickr.photosets.getPhotos(photoset_id=config['album_id'], page=str(page))
        all_photos.extend(photo["id"] for photo in resp['photoset']['photo'])
        if int(resp['photoset']['page']) == resp['photoset']['pages']:
            # we're right in the last page
            break
        page += 1
    all_photos.remove(config['persistent_photo_id'])
    logger.info("Found %d photos", len(all_photos))

    for num, photo_id in enumerate(all_photos, 1):
        logger.info("Downloading photo id=%s (#%d)", photo_id, num)
        resp = flickr.photos.getSizes(photo_id=photo_id)
        for size_data in resp['sizes']['size']:
            if size_data['label'] == 'Original':
                photo_url = size_data['source']
                break
        else:
            logger.error("Original size not found for photo id %s", photo_id)
            exit()

        # download image to original image name
        http_resp = request.urlopen(photo_url)
        img_name = photo_url.split("/")[-1]
        filepath = os.path.join(dirpath, img_name)
        with open(filepath, "wb") as fh:
            while True:
                data = http_resp.read(65536)
                if not data:
                    break
                fh.write(data)

        # rename using a prefix with own photo timestamp
        with open(filepath, "rb") as fh:
            tags = exifread.process_file(fh)
        if tags:
            try:
                exif_dt = tags['EXIF DateTimeOriginal'].values
            except KeyError:
                logger.warning("EXIF entry not found for %r (%s)", filepath, tags.keys())
                prefix = "unknown-datetime"
            else:
                dt = datetime.datetime.strptime(exif_dt, "%Y:%m:%d %H:%M:%S")
                prefix = dt.strftime("%Y%m%d-%H%M%S")
        else:
            logger.warning("Tags not found for %r", filepath)
            prefix = "unknown-datetime"
        new_filepath = os.path.join(dirpath, prefix + "-" + img_name)
        os.rename(filepath, new_filepath)

        # removing from server
        logger.info("Removing remote")
        resp = flickr.photos.delete(photo_id=photo_id)
        if resp.get('stat') != 'ok':
            logger.warning("Bad response", resp)

    logger.info("Done.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("config", help="The configuration file")
    parser.add_argument("path", help="The destination for the downloaded images.")
    parser.add_argument("--quiet", help="Don't produce output", action='store_true')
    args = parser.parse_args()

    if args.quiet:
        logger.setLevel(logging.WARNING)
    if not os.path.exists(args.path):
        os.makedirs(args.path)

    main(args.config, args.path)
